#include "bezier.h"
#include <boost/lexical_cast.hpp>

Bezier::Bezier(std::string nome, std::vector<Coordenada> pontos, Cor cor ): Curva(nome, pontos, cor)
{
}

std::vector<Curva::Polinomio> Bezier::obterPolinomios(){
	std::vector<Curva::Polinomio> retorno;
	//TODO Usar iterador
	Matriz<4,4> base;
	base[0][0] = -1;	base[0][1] = +3;	base[0][2] = -3;	base[0][3] = +1;
	base[1][0] = +3;	base[1][1] = -6;	base[1][2] = +3;	base[1][3] =  0;
	base[2][0] = -3;	base[2][1] = +3;	base[2][2] =  0;	base[2][3] =  0;
	base[3][0] = +1;	base[3][1] =  0;	base[3][2] =  0;	base[3][3] =  0;
	for(unsigned i = 0; i+3 < _coordenadas.size(); i += 3){
		Curva::Polinomio nova;
                Matriz<4,3> pontos;
                pontos[0][0] = _coordenadas[ i ].x();	pontos[0][1] = _coordenadas[ i ].y();   pontos[0][2] = _coordenadas[ i ].z();
                pontos[1][0] = _coordenadas[i+1].x();	pontos[1][1] = _coordenadas[i+1].y();   pontos[1][2] = _coordenadas[i+1].z();
                pontos[2][0] = _coordenadas[i+2].x();	pontos[2][1] = _coordenadas[i+2].y();   pontos[2][2] = _coordenadas[i+2].z();
                pontos[3][0] = _coordenadas[i+3].x();	pontos[3][1] = _coordenadas[i+3].y();   pontos[3][2] = _coordenadas[i+3].z();

		nova = base*pontos;

		retorno.push_back(nova);
    }
	return retorno;
}

std::string Bezier::tipo() const{
	return "curvaBezier";
}

Objeto* Bezier::clonar() const{
    return new Bezier(this->nome(), _coordenadas, this->cor());
}

#include <iostream>
void Bezier::persist(TiXmlNode * where) const {
	TiXmlElement* me = createTag();
	saveNameAndColor(me);
	for(std::vector<Coordenada>::const_iterator i = _coordenadas.begin(); i != _coordenadas.end(); ++i){
		TiXmlElement* geometria = new TiXmlElement("geometria");
		std::vector<Coordenada>::const_iterator first = i;
		++i;
		if(i == _coordenadas.end()) break;
		++i;
		if(i == _coordenadas.end()) break;
		++i;
		if(i == _coordenadas.end()) break;
		saveCoordenadasIdentificadas(geometria, first, i+1);
		--i;
		me->LinkEndChild(geometria);
	}
	where->LinkEndChild(me);
}
void Bezier::saveCoordenadasIdentificadas(TiXmlElement *me, iterator i, iterator end) const {
	int contador = 0;
	std::string base = "p";
	for(; i != end; ++i) {
		contador++;
		std::string tagAtual = base;
		tagAtual += boost::lexical_cast<std::string>(contador);
		TiXmlElement* eachPoint = new TiXmlElement( tagAtual );
		i->persist(eachPoint);
		me->LinkEndChild(eachPoint);
	}
}

void Bezier::restore(TiXmlElement * who) {
	Objeto::restore(who);
	TiXmlNode* geometria = 0;

	TiXmlNode* primeiroNodo;
	if(	(primeiroNodo = who->FirstChild("geometria")) != NULL ) {
		primeiroNodo = primeiroNodo->FirstChild("p1");
		TiXmlElement* primeiroPonto = primeiroNodo->ToElement();
		Coordenada newCoordenada;
		newCoordenada.restore(primeiroPonto);
		_coordenadas.push_back(newCoordenada);
	}
	while( (geometria = who->IterateChildren("geometria", geometria	))){
		TiXmlNode* cadaPonto = 0;

		cadaPonto = geometria->FirstChild("p2");
		{
		TiXmlElement* actualCoordenada = cadaPonto->ToElement();
		Coordenada newCoordenada;
		newCoordenada.restore(actualCoordenada);
		_coordenadas.push_back(newCoordenada);
		}

		cadaPonto = geometria->FirstChild("p3");
		{
		TiXmlElement* actualCoordenada = cadaPonto->ToElement();
		Coordenada newCoordenada;
		newCoordenada.restore(actualCoordenada);
		_coordenadas.push_back(newCoordenada);
		}

		cadaPonto = geometria->FirstChild("p4");
		{
		TiXmlElement* actualCoordenada = cadaPonto->ToElement();
		Coordenada newCoordenada;
		newCoordenada.restore(actualCoordenada);
		_coordenadas.push_back(newCoordenada);
		}

	}

}

class FabricaDeBezier : public FabricaDeObjeto {
public:
	~FabricaDeBezier(){
	};
	std::string label() const{
		return "Bezier";
	}
	char quantidadeValidaDePontos(int quantidade) const{
		return ( (quantidade-4)%3 == 0 && quantidade >= 4) ? 0 : -1;
	}
	Objeto* criar() const{
		std::string nome = ((String*)_opcoes.at(0))->value;
		std::vector<Coordenada> coordenadas = ((Coordenadas*)_opcoes.at(1))->value;
		Cor cor = ((Color*)_opcoes.at(2))->value;
		return new Bezier(nome, coordenadas, cor);
	}

	virtual Objeto* restore(TiXmlElement * who) const {
		Objeto* objeto = new Bezier("", std::vector<Coordenada>());
		objeto->restore(who);

		return objeto;
	}
};
FabricaDeBezier unicaFabricaDeBezier;

FabricaDeObjeto* obterFabricaDeBezier(){
	return &unicaFabricaDeBezier;
}



BEGIN_FACTORY_REGISTER(bezier)
	FabricaDeObjeto::fabricas["curvaBezier"] = obterFabricaDeBezier();
END_FACTORY_REGISTER(bezier)

