#include "clipping/estrategiapuel.h"
#include "clipping/estrategiasutherland.h"

std::vector<Coordenada> EstrategiaPuel::clipar(const std::vector<Coordenada>& coordenadas, Window* clipador) const{
	using std::vector;

	if(coordenadas.size() < 2)
		return coordenadas;

	vector<Coordenada> retorno;
	vector<Coordenada>::const_iterator i=coordenadas.begin();
	EstrategiaSutherland estrategiaReta;

	//ESTADO INICIAL
	char regiao = regiaoExclusiva(*i, clipador);
	if(regiao == 0){
		retorno.push_back(*i);
	}

	for(++i; i != coordenadas.end(); ++i){
		char novaRegiao = regiaoExclusiva(*i, clipador);
		//ESTADO DENTRO
		if(regiao == 0){
			//CONTINUAR
			if(novaRegiao == 0){
				retorno.push_back(*i);
			}

			//SAIR
			else{
				vector<Coordenada> clipadas = estrategiaReta.clipar(*(i-1), *i, clipador);
				retorno.push_back(clipadas[1]);

				//SAIR E MUDA DE REGIAO
				char regiaoNaBorda = regiaoExclusivaNaBorda(clipadas[1], clipador);
				if(novaRegiao != regiaoNaBorda){
					retorno.push_back( puxar(*i, novaRegiao|regiaoNaBorda, clipador) );
				}
			}
		}
		//ESTADO FORA
		else{
			//ENTRAR
			if(novaRegiao == 0){
				vector<Coordenada> clipadas = estrategiaReta.clipar(*(i-1), *i, clipador);
				//ENTRAR E MUDA DE REGIAO
				char regiaoNaBorda = regiaoExclusivaNaBorda(clipadas[0], clipador);
				if(regiao != regiaoNaBorda){
					retorno.push_back( puxar(*i, regiao|regiaoNaBorda, clipador) );
				}
				retorno.push_back(clipadas[0]);
				retorno.push_back(*i);
			}
			//TROCAR DE REGIAO
			else if(regiao != novaRegiao){
				vector<Coordenada> clipadas = estrategiaReta.clipar(*(i-1), *i, clipador);
				if(clipadas.size() > 0){
					retorno.push_back(clipadas[0]);
					retorno.push_back(clipadas[1]);
				}else{
					//TROCAR DE REGIAO PARA CONTRARIA
					if(regiao == regiaoContraria(novaRegiao)){
						retorno.push_back(  puxar( *(i-1), estrategiaReta.obterRegiao(*(i-1), clipador), clipador )  );
						retorno.push_back(  puxar( *i, estrategiaReta.obterRegiao(*(i), clipador), clipador )  );
					}
					//TROCAR DE REGIAO NORMAL
					else{
						retorno.push_back( puxar(*i, regiao|novaRegiao, clipador) );
					}
				}
			}
		}

		regiao = regiaoExclusiva(*i, clipador);

	};
	return retorno;
}

char EstrategiaPuel::regiaoExclusiva(const Coordenada& coordenada, Window* clipador) const{
	double x0=-(clipador->viewRightVector().modulo()/2.0), x1=clipador->viewRightVector().modulo()/2.0,
		y0=-(clipador->viewUpVector().modulo()/2.0), y1=clipador->viewUpVector().modulo()/2.0,
		x=coordenada.x(),y=coordenada.y();

	if( y > y1 && y-y1 > x0-x && y-y1 > x-x1 )
		return TOP;

	if( y < y0 && y0-y > x0-x && y0-y > x-x1 )
		return BOTTOM;

	if( x > x1 )
		return RIGHT;

	if( x < x0 )
		return LEFT;

	return 0;
}
char EstrategiaPuel::regiaoExclusivaNaBorda(const Coordenada& coordenada, Window* clipador) const{
	double x0=-(clipador->viewRightVector().modulo()/2.0), x1=clipador->viewRightVector().modulo()/2.0,
		y0=-(clipador->viewUpVector().modulo()/2.0), y1=clipador->viewUpVector().modulo()/2.0,
		x=coordenada.x(),y=coordenada.y();

	if( y-y1 > x0-x && y-y1 > x-x1 )
		return TOP;

	if( y0-y > x0-x && y0-y > x-x1 )
		return BOTTOM;

	if( x > 0 )
		return RIGHT;

	if( x < 0 )
		return LEFT;

	return 0;
}

Coordenada EstrategiaPuel::puxar(const Coordenada & coordenada, int regiao, Window * clipador) const{
	Coordenada retorno = coordenada;
	if(regiao & TOP){
		retorno.y( clipador->viewUpVector().modulo()/2.0 );
	}else if(regiao & BOTTOM){
		retorno.y( -clipador->viewUpVector().modulo()/2.0 );
	}
	if(regiao & LEFT){
		retorno.x( -clipador->viewRightVector().modulo()/2.0);
	}else if(regiao & RIGHT){
		retorno.x( clipador->viewRightVector().modulo()/2.0);
	}
	return retorno;
}

char EstrategiaPuel::regiaoContraria(char regiao) const{
	//TODO Otmizar isso daqui usando bit-hacks:
	char nova=0;
	if(regiao & TOP){
		nova |= BOTTOM;
	}else if(regiao & BOTTOM){
		nova |= TOP;
	}
	if(regiao & LEFT){
		nova |= RIGHT;
	}else if(regiao & RIGHT){
		nova |= LEFT;
	}
	return nova;
}
