#include "clipping/estrategiasutherland.h"

//#define TOLERANCIA 0.0000001
#define TOLERANCIA 0.0001

std::vector<Coordenada>
		EstrategiaSutherland::clipar(const Coordenada& origem, const Coordenada& fim, Window* clipador) const{
	char regiaoInicio, regiaoFim;

	regiaoInicio = obterRegiao(origem, clipador);
	regiaoFim = obterRegiao(fim, clipador);

	std::vector<Coordenada> retorno;
	if( (regiaoInicio|regiaoFim) == 0){
		retorno.push_back(origem);
		retorno.push_back(fim);
		return retorno;
	}

	if( regiaoInicio & regiaoFim ){
		return retorno;
	}

	Coordenada novaOrigem = origem;
	Coordenada novoFim = fim;
	if( !puxar(novaOrigem, origem, fim, regiaoInicio, clipador) || !puxar(novoFim, origem, fim, regiaoFim, clipador) ){
		return retorno;
	}

	retorno.push_back(novaOrigem);
	retorno.push_back(novoFim);
	return retorno;
}

char EstrategiaSutherland::obterRegiao(const Coordenada& coordenada, Window * clipador) const{
	char regioes = 0;

	if(coordenada.y()-TOLERANCIA > clipador->viewUpVector().modulo()/2.0){
		regioes |= TOP;
	}else if(coordenada.y()+TOLERANCIA < -clipador->viewUpVector().modulo()/2.0){
		regioes |= BOTTOM;
	}
	if(coordenada.x()-TOLERANCIA > clipador->viewRightVector().modulo()/2.0){
		regioes |= RIGHT;
	}else if(coordenada.x()+TOLERANCIA < -clipador->viewRightVector().modulo()/2.0){
		regioes |= LEFT;
	}
	return regioes;
}

bool EstrategiaSutherland::puxar
		(Coordenada & coordenada, const Coordenada& inicio, const Coordenada& fim, char regiao, Window * clipador) const{
	using namespace std;
	if(regiao & TOP){
		coordenada.x( xEm(clipador->viewUpVector().modulo()/2.0,inicio,fim) );
		coordenada.y( clipador->viewUpVector().modulo()/2.0 );
		if( obterRegiao(coordenada, clipador) == 0 )
			return true;
	}else if(regiao & BOTTOM){
		coordenada.x( xEm( -clipador->viewUpVector().modulo()/2.0,inicio,fim ) );
		coordenada.y( -clipador->viewUpVector().modulo()/2.0 );
		if( obterRegiao(coordenada, clipador) == 0 )
			return true;
	}
	if(regiao & LEFT){
		coordenada.x( -clipador->viewRightVector().modulo()/2.0 );
		coordenada.y( yEm(-clipador->viewRightVector().modulo()/2.0,inicio,fim) );
		return obterRegiao(coordenada, clipador) == 0;
	}else if(regiao & RIGHT){
		coordenada.x( clipador->viewRightVector().modulo()/2.0 );
		coordenada.y( yEm(clipador->viewRightVector().modulo()/2.0,inicio,fim) );
		return obterRegiao(coordenada, clipador) == 0;
	}
	return true;
}

double EstrategiaSutherland::a(const Coordenada &p0, const Coordenada &p1) const{
	return (p1.y()-p0.y())/(p1.x()-p0.x());
}
double EstrategiaSutherland::b(const Coordenada &p0, const Coordenada &p1) const{
	return p0.y() - a(p0,p1)*p0.x();
}
double EstrategiaSutherland::xEm(double y, const Coordenada &p0, const Coordenada &p1) const{
	if(p0.x() == p1.x() )
		return p0.x();
	if(a(p0,p1) == 0.0)
		return p0.y();
	return (y-b(p0,p1))/a(p0,p1);
}
double EstrategiaSutherland::yEm(double x, const Coordenada &p0, const Coordenada &p1) const{
	if(p0.x() == p1.x() )
		return p0.y();
	return a(p0,p1)*x + b(p0,p1);
}
