#ifndef __POLIMORFICITERATOR_H_
#define __POLIMORFICITERATOR_H_

/*
 * TODO O conceito de interface para o iterator é boa
 * mas preciso amadurecer um pouco a idéia de como implementar
 * isso, por enquanto não vai ser usado.
 */

namespace libcg {
namespace container {

template <typename T>
class Iterator {
public:
	typedef int difference_type;
public:
	virtual ~Iterator(){};
	virtual Iterator& operator++()=0;
	virtual Iterator operator++(int)=0;
	virtual Iterator& operator--()=0;
	virtual Iterator operator--(int)=0;
	virtual Iterator operator+(difference_type) const =0;
	virtual Iterator operator-(difference_type) const =0;
	virtual Iterator& operator=(const Iterator&)=0;
	virtual T& operator*()=0;
	virtual T* operator->()=0;
	virtual Iterator* clone() const { return &this; };
};

template <typename T, typename IteratorType>
class AutomaticIterator : public Iterator<T> {
public:
	typedef int difference_type;
public:
	AutomaticIterator(const Iterator<T>& it){
		it = value;
	};

	AutomaticIterator& operator++(){
		++value;
		return *this;
	};

	AutomaticIterator operator++(int){
		return AutomaticIterator(value++);
	};

	AutomaticIterator& operator--(){
		--value;
		return *this;
	};

	AutomaticIterator operator--(int){
		return value--;
	};

	AutomaticIterator operator+(difference_type delta) const {
		return value + delta;
	};

	AutomaticIterator operator-(difference_type delta) const {
		return value - delta;
	};

	AutomaticIterator& operator=(const Iterator<T>& that){
		value = that;
		return *this;
	};

	T& operator*(){
		return *value;
	}

	T* operator->(){
		return value.operator->();
	}

	Iterator<T>* clone() const {
		return new AutomaticIterator(value);
	};
private:
	IteratorType value;
};

template <typename T>
class PolimorficIterator {
public:
	typedef int difference_type;
public:
	PolimorficIterator(Iterator<T>* it){
		value = it;
	}
	PolimorficIterator(const Iterator<T>& it){
		value = it.clone();
	}
	virtual ~PolimorficIterator(){
		delete value;
	}

	template <class Iterator>
	PolimorficIterator(const Iterator& it){
		value = new AutomaticIterator<T,Iterator>(it);
	}

	inline PolimorficIterator& operator++(){
		return ++(*value);
	};

	inline PolimorficIterator operator++(int){
		return (*value)++;
	}
	inline PolimorficIterator& operator--(){
		return --(*value);
	}
	inline PolimorficIterator operator--(int){
		return (*value)--;
	}
	inline PolimorficIterator operator+(difference_type delta) const{
		return (*value)+delta;
	}
	inline PolimorficIterator operator-(difference_type delta) const{
		return (*value)-delta;
	}
	inline Iterator<T>& operator=(const Iterator<T>& that){
		return ((*value)=that);
	}
	inline T& operator*(){
		return *(*value);
	};
	inline T* operator->(){
		return value->operator->();
	};
private:
	Iterator<T>* value;
};



}
}

#endif/*__POLIMORFICITERATOR_H_*/
