#include "coordenada.h"
#include <limits>

Coordenada::Coordenada(){
	(*this)[0][0] = std::numeric_limits<double>::infinity();
	(*this)[0][1] = std::numeric_limits<double>::infinity();
	(*this)[0][2] = std::numeric_limits<double>::infinity();
	(*this)[0][3] = 1;
}

bool Coordenada::isNull() const{
	return (*this)[0][0] == std::numeric_limits<double>::infinity() ||
	       (*this)[0][1] == std::numeric_limits<double>::infinity() ||
	       (*this)[0][2] == std::numeric_limits<double>::infinity();
}

Coordenada::Coordenada(double x, double y, double z){
	(*this)[0][0] = x;
	(*this)[0][1] = y;
	(*this)[0][2] = z;
	(*this)[0][3] = 1;
}

Coordenada::Coordenada(const Matriz<1, COORDENADA_LARGURA> & outra): Matriz<1,COORDENADA_LARGURA>(outra){
}

Coordenada::Coordenada(const Coordenada& coordenada): Matriz<1,COORDENADA_LARGURA>(){
	(*this)[0][0] = coordenada.x();
	(*this)[0][1] = coordenada.y();
	(*this)[0][2] = coordenada.z();
	(*this)[0][3] = 1;
}

Coordenada::~Coordenada(){
}

void Coordenada::persist(TiXmlElement * who) const {
	who->SetDoubleAttribute("x", (*this)[0][0]);
	who->SetDoubleAttribute("y", (*this)[0][1]);
	who->SetDoubleAttribute("z", (*this)[0][2]);
}
void Coordenada::restore(TiXmlElement * who) {
	double novoX;
	double novoY;
	double novoZ;
	if ( who->QueryDoubleAttribute("x", &novoX) != TIXML_SUCCESS ||
		 who->QueryDoubleAttribute("y", &novoY) != TIXML_SUCCESS ||
		 who->QueryDoubleAttribute("z", &novoZ) != TIXML_SUCCESS )
	{
		throw std::string("Coordenada sem x, sem y ou sem z.");
	}
	(*this)[0][0] = novoX;
	(*this)[0][1] = novoY;
	(*this)[0][2] = novoZ;
}

double Coordenada::x() const{
	return (*this)[0][0];
}

double Coordenada::y() const{
	return (*this)[0][1];
}

double Coordenada::z() const{
	return (*this)[0][2];
}

void Coordenada::x(double novoX){
	(*this)[0][0] = novoX;
}

void Coordenada::y(double novoY){
	(*this)[0][1] = novoY;
}

void Coordenada::z(double novoZ){
	(*this)[0][2] = novoZ;
}

Coordenada Coordenada::operator+(const Coordenada& that) const{
	return Coordenada(this->x() + that.x(), this->y() + that.y(), this->z() + that.z());
}

Coordenada Coordenada::operator-(const Coordenada& that) const{
	return Coordenada(this->x() - that.x(), this->y() - that.y(), this->z() - that.z());
}

Coordenada Coordenada::operator*(double escalar) const{
	return Coordenada(this->x()*escalar, this->y()*escalar, this->z()*escalar);
}
bool Coordenada::operator==(const Coordenada& that) const{
	return this->x() == that.x() && this->y() == that.y() && this->z() == that.z();
}
