#include "curva3d.h"
#include "window.h"

#include "clipping/estrategianear.h"

Curva3D::Curva3D(std::string nome, std::vector<Geometria> geometrias, Cor cor): ObjetoClipavel(nome, cor) {
    this->geometrias = geometrias;
	_nivelDePreenchimento = 1;
}

Curva3D::~Curva3D() {
}

void Curva3D::aplicarTransformacao(const Transformacao& transformacao){
	for(std::vector<Geometria>::iterator i = geometrias.begin(); i != geometrias.end(); ++i){
		for(int k = 0; k < 4; ++k)
			for(int j = 0; j < 4; ++j)
				(*i)[k][j] *= transformacao;
	}
}

#define min(a,b) ( (a) < (b) ? (a) : (b) )
void Curva3D::recalcularCache(Window* clipador){
    std::vector<Coordenada> retas;
	double step = min(clipador->viewUpVector().modulo()/500.0, 1/5.0);
//	double step = 1.0/3.0;
        for(std::vector<Geometria>::iterator i= geometrias.begin(); i != geometrias.end(); ++i){
            std::vector<Matriz<4,4> > parametros = aplicarMatriz(*i);
			Matriz<4,4> parametrosX = parametros [0];
			Matriz<4,4> parametrosY = parametros [1];
			Matriz<4,4> parametrosZ = parametros [2];
			Matriz<1,4> ms;
			Matriz<4,1> mt;

            for(double t=step; t < 1.0; t+=step){
				mt[0][0] = t*t*t;
				mt[0][1] = t*t;
				mt[0][2] = t;
				mt[0][3] = 1.0;
				Matriz<4,1> oiX = parametrosX * mt;
				Matriz<4,1> oiY = parametrosY * mt;
				Matriz<4,1> oiZ = parametrosZ * mt;

                for(double s=step; s < 1.0; s+= step){
					ms[0][0] = 1.0;
					ms[1][0] = s;
					ms[2][0] = s*s;
					ms[3][0] = s*s*s;
//					double x=0.0, y=0.0, z=0.0;
//					for(int k = 0; k < 4; k++){
//						x += oiX[0][k] * ms[k][0];
//						y += oiY[0][k] * ms[k][0];
//						z += oiZ[0][k] * ms[k][0];
//					}
					double x = (oiX * ms)[0][0];
					double y = (oiY * ms)[0][0];
					double z = (oiZ * ms)[0][0];
					retas.push_back(Coordenada(x,y,z));
                }
            }
        }
		_coordenadasClipadas = aplicarPerspectiva(EstrategiaNear().clipar(retas, clipador), clipador);
}

std::string Curva3D::tipo() const {
    return "Curva3D";
}
