#ifndef FABRICADEOBJETO_H
#define FABRICADEOBJETO_H

#include "objeto.h"
#include <string>
#include <vector>
#include <QWidget>
#include "cor.h"
#include <map>

class FabricaDeObjeto
{
public:
	enum Types {
		COORDENADAS,
		COLOR,
		STRING,
		BOOLEAN,
		SELECTION,
		INT_INT
	};
#define TYPE(x) Types getType() const { return x; }
	
	class Type {
	public:
		virtual Types getType() const=0;
                virtual bool fulfiled() const { return true; }
		typedef std::string Label;
		Label label;
	};

	class Coordenadas : public Type {
	friend class FabricaDeObjeto;
	public:
                TYPE(COORDENADAS)
		typedef std::vector<Coordenada> Value;
		Value value;
		virtual bool fulfiled() const { return enoughQuantity() > -1; };
		virtual int enoughQuantity() const {
			return owner->quantidadeValidaDePontos(value.size());
		}

	private:
		FabricaDeObjeto* owner;
	};
	class Color : public Type  {
	public:
                TYPE(COLOR)
		typedef Cor Value;
		Value value;
	};

	class String : public Type  {
	public:
                TYPE(STRING)
		typedef std::string Value;
		Value value;
		virtual bool fulfiled() const { return value.size() > 0; };
	};

	class Boolean : public Type  {
	public:
                TYPE(BOOLEAN)
		typedef bool Value;
		Value value;
	};

	class Selection : public Type  {
	public:
                TYPE(SELECTION)
		typedef unsigned int Value;
		std::vector<std::string> options;
		Value value;
	};

	class IntInt : public Type  {
	public:
                TYPE(INT_INT)
		typedef std::vector<std::pair<int,int> > Value;
		Value value;
	};

	typedef std::vector<Type*> Options;
public:
        typedef std::map<std::string, FabricaDeObjeto*> Mapa;
        typedef Mapa::iterator iterator;
        static Mapa fabricas;
public:
	FabricaDeObjeto();
	virtual ~FabricaDeObjeto();
	virtual std::string label() const=0;
	virtual Options opcoes();
	/**
	  * @return -1 se não tem pontos suficientes, se tem pontos suficientes: 1 se não aceita mais pontos e 0 se ainda aceita pontos.
	  */
	virtual char quantidadeValidaDePontos(int) const { return 1; }

	virtual Objeto* criar() const=0;
	virtual Objeto* restore(TiXmlElement*) const {
		throw "Restore não reimplementado!";
	}
protected:
	Options _opcoes;
};

#define BEGIN_FACTORY_REGISTER(x) class _factory_register_##x { \
    public: \
	_factory_register_##x() {
#define END_FACTORY_REGISTER(x) } } _factory_register_##x##_call;

#endif // FABRICADEOBJETO_H
