#include "objeto.h"
#include "window.h"
#include "clipping/estrategianear.h"

Objeto::Objeto(std::string nome, Cor cor): _nome(nome), _cor(cor){
}

Objeto::~Objeto(){
}

const std::string Objeto::nome() const{
	return _nome;
}

void Objeto::aplicarTransformacao(const Transformacao& transformacao){
	for(std::vector<Coordenada>::iterator i = _coordenadas.begin(); i != _coordenadas.end(); i++)
		*i *= transformacao;
}

void Objeto::persist(TiXmlNode * where) const {
	TiXmlElement* me = createTag();
	saveNameAndColor(me);
	saveCoordenadas(me, coordenadasBegin(), coordenadasEnd());
	where->LinkEndChild(me);
}
TiXmlElement* Objeto::createTag() const {
	std::string tag = this->tipo();
	TiXmlElement* me = new TiXmlElement( tag.c_str() );
	return me;
}

void Objeto::saveNameAndColor(TiXmlElement *me) const {
	me->SetAttribute("nome", this->nome().c_str());
	me->SetAttribute("cor", this->cor().hex().c_str());
}
void Objeto::saveCoordenadas(TiXmlElement *me, iterator i, iterator end) const {
	for(; i != end; ++i) {
		TiXmlElement* eachPoint = new TiXmlElement( "ponto" );
		i->persist(eachPoint);
		me->LinkEndChild(eachPoint);
	}
}

void Objeto::restore(TiXmlElement * who) {
	const char* nome = who->Attribute("nome");
	const char* cor = who->Attribute("cor");
	this->nome( nome == NULL ? "Sem Nome" : std::string(nome) );
	this->_cor = Cor( std::string(
			cor == NULL ? "000000" : cor
			));

	TiXmlNode* child = 0;

	_coordenadas.clear();
	while( (child = who->IterateChildren("ponto", child)) ) {
		TiXmlElement* actualCoordenada = child->ToElement();
		Coordenada newCoordenada;
		newCoordenada.restore(actualCoordenada);
		_coordenadas.push_back(newCoordenada);
	}

}

#include <iostream>
std::vector<Coordenada> Objeto::aplicarPerspectiva(std::vector<Coordenada> coordenada, Window * clipador){
	std::vector<Coordenada> retorno;
	EstrategiaNear cortarNegativos;
	coordenada = cortarNegativos.clipar(coordenada, clipador);
	for(std::vector<Coordenada>::iterator i = coordenada.begin(); i != coordenada.end(); i++){
		if(i->isNull()){
			retorno.push_back(*i);
			continue;
		}
		Coordenada linguisa;
		linguisa.x( i->x()* clipador->near() / (i->z()) );
		linguisa.y( i->y()* clipador->near() / (i->z()) );
		linguisa.z( i->z() );
		retorno.push_back(linguisa);
	}
	return retorno;
}



void Objeto::nome(const std::string nome){
	_nome = nome;
}

Coordenada Objeto::centro() const {
	Coordenada retorno(0.0,0.0,0.0);
    for(std::vector<Coordenada>::const_iterator i = _coordenadas.begin(); i != _coordenadas.end(); i++){
		retorno = retorno + (*i);
    }
    return Coordenada(retorno.x()/_coordenadas.size(), retorno.y()/_coordenadas.size(), retorno.z()/_coordenadas.size());

}
Cor Objeto::cor() const {
    return _cor;
}

void Objeto::cor(const Cor& novaCor){
    _cor = novaCor;
}

std::string Objeto::tipo() const {
	return "objeto";
};

Objeto* Objeto::clonar() const {
	return new Objeto(this->nome(), this->cor());
}

char Objeto::nivelDePreenchimento() const {
	return _nivelDePreenchimento;
}

Objeto::iterator Objeto::coordenadasBegin() const {
	return _coordenadas.begin();
}

Objeto::iterator Objeto::coordenadasEnd() const {
	return _coordenadas.end();
}

Objeto::iterator Objeto::coordenadasBegin(Window*){
	return _coordenadas.begin();
}

Objeto::iterator Objeto::coordenadasEnd(Window*) const {
	return _coordenadas.end();
}

int Objeto::coordenadasSize() const {
	return _coordenadas.size();
}

int Objeto::coordenadasSize(Window*) {
	return _coordenadas.size();
}

Objeto::Range Objeto::coordenadas() const{
	iterator begin = coordenadasBegin();
	return Range(begin, coordenadasEnd());
}

Objeto::Range Objeto::coordenadas(Window* clipador){
	iterator begin = coordenadasBegin(clipador);
	return Range(begin, coordenadasEnd(clipador));
}
