#ifndef __OBJETO_H
#define __OBJETO_H
class Objeto;

#include <string>
#include <vector>
#include "coordenada.h"
#include "cor.h"
#include "transformacao.h"
#include "tinyxml/tinyxml.h"
class Window;//#include "window.h"

//TODO namespace libcg {

class Objeto{
public:
	typedef std::vector<Coordenada>::const_iterator iterator;
	typedef std::pair<iterator,iterator> Range;
public:
	Objeto(std::string nome, Cor cor);
	virtual	~Objeto();

	virtual void persist(TiXmlNode*) const;
	virtual void restore(TiXmlElement*);
protected:
	void saveNameAndColor(TiXmlElement*) const;
	void saveCoordenadas(TiXmlElement *me, iterator i, iterator end) const;
	TiXmlElement* createTag() const;
public:
	
	const std::string nome() const;
	void nome(const std::string novoNome);
	virtual void aplicarTransformacao(const Transformacao&);
	std::vector<Coordenada> aplicarPerspectiva(std::vector<Coordenada>, Window*);

	virtual iterator coordenadasBegin() const;
	virtual int coordenadasSize() const;
	virtual iterator coordenadasEnd() const;

	/**
	  * O begin SEMPRE deve ser chamado antes do end.
	  */
	virtual iterator coordenadasBegin(Window* clipador);
	virtual int coordenadasSize(Window* clipador);
	virtual iterator coordenadasEnd(Window* clipador) const;

	virtual Range coordenadas() const;
	virtual Range coordenadas(Window* clipador);

	virtual Coordenada centro() const;
	virtual std::string tipo() const;

	Cor cor() const;
	void cor(const Cor&);

	virtual Objeto* clonar() const;

	char nivelDePreenchimento() const;
private:
	std::string _nome;
	Cor _cor;
protected:
	std::vector<Coordenada> _coordenadas;
	char _nivelDePreenchimento;
};

#endif
