#include "objeto3d.h"
#include "clipping/estrategianear.h"
#include "clipping/estrategiapoligonoviaretas.h"

#include <iostream>
Objeto3D::Objeto3D(std::string nome, Cor cor, std::vector<Coordenada> vertices, Arestas arestas, bool wireframe): ObjetoClipavel(nome, cor)
{
    _coordenadas = vertices;
    this->arestas = arestas;
    _nivelDePreenchimento = wireframe? 1 : 2;
}

Objeto3D::~Objeto3D(){
}

void Objeto3D::persist(TiXmlNode * where) const {
	TiXmlElement* me = createTag();
	saveNameAndColor(me);
	TiXmlElement* pontos = new TiXmlElement("pontos");
	saveCoordenadas(pontos, coordenadasBegin(), coordenadasEnd());
	me->LinkEndChild(pontos);

	TiXmlElement* arestas = new TiXmlElement("arestas");

	for(Arestas::const_iterator i = this->arestas.begin(); i != this->arestas.end(); ++i) {
		int first=-1;
		int second=-1;
		for(int j=0; j < _coordenadas.size(); ++j) {
			if( _coordenadas[j] == i->first) {
				first = j;
			}
			if( _coordenadas[j] == i->second) {
				second = j;
			}
		}
		if(first != -1 && second != -1){
			TiXmlElement* aresta = new TiXmlElement("aresta");
			aresta->SetAttribute("v1", first);
			aresta->SetAttribute("v2", second);
			arestas->LinkEndChild(aresta);
		}
	};
	me->LinkEndChild(arestas);
	where->LinkEndChild(me);
}
#include <iostream>
void Objeto3D::restore(TiXmlElement * who) {
	Objeto::restore(who);
	TiXmlNode* pontosNode = who->FirstChild("pontos");
	TiXmlNode* arestasNode = who->FirstChild("arestas");
	if(pontosNode == NULL || arestasNode == NULL ) {
		throw std::string("Os pontos ou as arestas do objeto 3D não foram definidas.");
	}
	TiXmlElement* pontos = pontosNode->ToElement();
	TiXmlElement* arestas = arestasNode->ToElement();

	TiXmlNode* child=0;
	_coordenadas.clear();
	while( (child = pontos->IterateChildren("ponto", child)) ) {
		TiXmlElement* actualCoordenada = child->ToElement();
		Coordenada newCoordenada;
		newCoordenada.restore(actualCoordenada);
		_coordenadas.push_back(newCoordenada);
	}
	this->arestas.clear();
	child=0;
	while( (child = arestas->IterateChildren("aresta", child)) ) {
		TiXmlElement* actualAresta = child->ToElement();
		int v1, v2;
		if( actualAresta->QueryIntAttribute("v1", &v1) == TIXML_SUCCESS &&
			actualAresta->QueryIntAttribute("v2", &v2) == TIXML_SUCCESS
			){
			this->arestas.push_back(Aresta( _coordenadas[v1], _coordenadas[v2] ));
		}else{
			std::cerr << "Warning: Linha " << actualAresta->Row() << " não contém uma aresta válida." << std::endl;
		}
	}
}

Objeto* Objeto3D::clonar() const{
    return new Objeto3D(this->nome(), this->cor(), _coordenadas, arestas, 1);
}

void Objeto3D::aplicarTransformacao(const Transformacao &transformacao){
    Objeto::aplicarTransformacao(transformacao);
    for(std::vector< std::pair<Coordenada, Coordenada> >::iterator i = arestas.begin(); i != arestas.end(); i++){
	    i->first *= transformacao;
	    i->second *= transformacao;
    }
}

void Objeto3D::recalcularCache(Window* clipador){
    _coordenadasClipadas.clear();
    for(std::vector<std::pair<Coordenada,Coordenada> >::iterator i = arestas.begin(); i != arestas.end(); i++){
		_coordenadasClipadas.push_back(i->first);
		_coordenadasClipadas.push_back(i->second);
		_coordenadasClipadas.push_back(Coordenada());
    }

    EstrategiaNear cortarNegativos;
    EstrategiaPoligonoViaRetas clipar;
    _coordenadasClipadas = clipar.clipar(aplicarPerspectiva(cortarNegativos.clipar(_coordenadasClipadas, clipador), clipador), clipador);
}

std::string Objeto3D::tipo() const{
	return "objeto3d";
}

class FabricaDeObjeto3D : public FabricaDeObjeto {
public:
	FabricaDeObjeto3D(){
		IntInt* arestas = new IntInt();
		arestas->label = "Arestas";
		_opcoes.push_back(arestas);
	}

	~FabricaDeObjeto3D(){
	};

	std::string label() const{
		return "Objeto3D";
	}
	char quantidadeValidaDePontos(int quantidade) const{
		return (quantidade < 1) ? -1 : 0;
	}
	Objeto* criar() const{
		typedef Objeto3D::Aresta Aresta;
		typedef Objeto3D::Arestas Arestas;
		typedef std::vector<std::pair<int,int> > ArestasPos;
		typedef ArestasPos::iterator PosIterator;


		std::string nome = ((String*)_opcoes.at(0))->value;
		std::vector<Coordenada> coordenadas = ((Coordenadas*)_opcoes.at(1))->value;
		Cor cor = ((Color*)_opcoes.at(2))->value;

		Arestas arestas;
		ArestasPos arestasPos = ((IntInt*)_opcoes.at(3))->value;
		for(PosIterator i = arestasPos.begin(); i != arestasPos.end(); ++i) {
			if(i->first >= coordenadas.size() || i->second >= coordenadas.size())
				return NULL;
			Coordenada origin = coordenadas[i->first];
			Coordenada dest = coordenadas[i->second];
			arestas.push_back( Aresta(origin, dest) );
		}

		return new Objeto3D(nome, cor, coordenadas, arestas,true);
	}
	virtual Objeto* restore(TiXmlElement * who) const {
		Objeto* objeto = new Objeto3D("", Cor(0,0,0), std::vector<Coordenada>(), Objeto3D::Arestas(), false);
		objeto->restore(who);
		return objeto;
	}
};
FabricaDeObjeto3D unicaFabricaDeObjeto3D;

FabricaDeObjeto* obterFabricaDeObjeto3D(){
	return &unicaFabricaDeObjeto3D;
}

BEGIN_FACTORY_REGISTER(objeto3D)
	FabricaDeObjeto::fabricas["objeto3d"] = obterFabricaDeObjeto3D();
END_FACTORY_REGISTER(objeto3D)
