#include "poligono.h"
#include "clipping/estrategiapuel.h"
#include "clipping/estrategiapoligonoviaretas.h"

#define TOLERANCIA 0.0000001

Poligono::Poligono(std::string nome, Cor cor): ObjetoClipavel(nome, cor) {
	_nivelDePreenchimento = 1;
	estrategia = new EstrategiaPoligonoViaRetas();
}

Poligono::Poligono(std::string nome, std::vector<Coordenada> coordenadas, Cor cor, bool preenchido): ObjetoClipavel(nome, cor)
{
    _coordenadas = coordenadas;
    if(_coordenadas.front() != _coordenadas.back())
        _coordenadas.push_back(_coordenadas.front());

	_nivelDePreenchimento = preenchido ? 2 : 1;

	if(_nivelDePreenchimento == 2)
		estrategia = new EstrategiaPuel();
	else
		estrategia = new EstrategiaPoligonoViaRetas();
}

void Poligono::persist(TiXmlNode * where) const {
	TiXmlElement* me = createTag();
	saveNameAndColor(me);
	if(coordenadasSize() > 0)
		saveCoordenadas(me, coordenadasBegin(), coordenadasEnd()-1);
	me->SetAttribute("cheio", _nivelDePreenchimento == 2 ? "true" : "false");
	where->LinkEndChild(me);
}
void Poligono::restore(TiXmlElement * who) {
	Objeto::restore(who);
	if(_coordenadas.front() != _coordenadas.back())
		_coordenadas.push_back(_coordenadas.front());

	const char* c_cheio = who->Attribute("false");
	if(c_cheio == NULL)
		return;
	std::string cheio(c_cheio);

	_nivelDePreenchimento = cheio == "true" ? 2 : 1;
}

Poligono::~Poligono(){
	delete estrategia;
}

std::string Poligono::tipo() const{
	return "poligono";
}

Coordenada Poligono::centro() const{
	Coordenada retorno(0.0,0.0,0.0);
    std::vector<Coordenada>::const_reverse_iterator i = _coordenadas.rbegin();
    for(i++; i != _coordenadas.rend(); i++){
	retorno = retorno + (*i);
    }
	return Coordenada(retorno.x()/(_coordenadas.size()-1), retorno.y()/(_coordenadas.size()-1), retorno.z()/(_coordenadas.size()-1));
}

Objeto* Poligono::clonar() const{
	return new Poligono(this->nome(), this->_coordenadas, this->cor(), _nivelDePreenchimento == 2);
}

void Poligono::recalcularCache(Window* clipador){
        _coordenadasClipadas = estrategia->clipar(aplicarPerspectiva(_coordenadas, clipador), clipador);
};

class FabricaDePoligono : public FabricaDeObjeto {
private:
public:
	FabricaDePoligono(){
		Boolean* sePreenchido = new Boolean();
		sePreenchido->label = "Preenchido";
		_opcoes.push_back(sePreenchido);
	}

	~FabricaDePoligono(){
		delete _opcoes.at(3);
	};
	std::string label() const{
		return "Poligono";
	}
	char quantidadeValidaDePontos(int quantidade) const {
		return (quantidade < 1) ? -1 : 0;
	}
	Objeto* criar() const{
		std::string nome = ((String*)_opcoes.at(0))->value;
		std::vector<Coordenada> coordenadas = ((Coordenadas*)_opcoes.at(1))->value;
		Cor cor = ((Color*)_opcoes.at(2))->value;
		bool sePreenchido = ((Boolean*)_opcoes.at(3))->value;
		return new Poligono(nome, coordenadas, cor, sePreenchido);
	}
	Objeto* restore(TiXmlElement * who) const {
		Objeto* objeto = new Poligono("");
		objeto->restore(who);

		return objeto;
	}
};
FabricaDePoligono unicaFabricaDePoligono;

FabricaDeObjeto* obterFabricaDePoligono(){
	return &unicaFabricaDePoligono;
}

BEGIN_FACTORY_REGISTER(poligono)
	FabricaDeObjeto::fabricas["poligono"] = obterFabricaDePoligono();
END_FACTORY_REGISTER(poligono)
