#include "reta.h"
#include "window.h"
#include "clipping/estrategiasutherland.h"
#include "clipping/estrategianear.h"

Reta::Reta(std::string nome, Coordenada origem, Coordenada destino, Cor cor): ObjetoClipavel(nome, cor){
	_coordenadas.clear();
	_coordenadas.push_back(origem);
	_coordenadas.push_back(destino);
	_nivelDePreenchimento = 1;
	_estrategia = new EstrategiaSutherland();
}

Reta::~Reta(){
	delete _estrategia;
}

void Reta::recalcularCache(Window* clipador){
	std::vector<Coordenada> clipadas = EstrategiaNear().clipar(_coordenadas, clipador);
	if(clipadas.size() == 2){
		_coordenadasClipadas = aplicarPerspectiva(_estrategia->clipar(clipadas[0], clipadas[1], clipador), clipador);
	}else{
		_coordenadasClipadas.clear();
	}
}


std::string Reta::tipo() const{
	return "reta";
}

Objeto* Reta::clonar() const{
	return new Reta(this->nome(), this->_coordenadas[0], this->_coordenadas[1], this->cor());
}

class FabricaDeReta : public FabricaDeObjeto {
public:
	~FabricaDeReta(){
	};
	std::string label() const{
		return "Reta";
	}
	char quantidadeValidaDePontos(int quantidade) const{
		return (quantidade < 2) ? -1 : 1;
	}
	Objeto* criar() const{
		std::string nome = ((String*)_opcoes.at(0))->value;
		std::vector<Coordenada> reta = ((Coordenadas*)_opcoes.at(1))->value;
		Cor cor = ((Color*)_opcoes.at(2))->value;
		return new Reta(nome, reta.at(0), reta.at(1), cor);
	}

	virtual Objeto* restore(TiXmlElement* who) const {
		Objeto* reta = new Reta("", Coordenada(), Coordenada());
		reta->restore(who);

		return reta;
	}

};
FabricaDeReta unicaFabricaDeReta;

FabricaDeObjeto* obterFabricaDeReta(){
	return &unicaFabricaDeReta;
}

BEGIN_FACTORY_REGISTER(reta)
	FabricaDeObjeto::fabricas["reta"] = obterFabricaDeReta();
END_FACTORY_REGISTER(reta)
