#include "poligono.h"
#include "clipping/estrategiasutherland.h"
#include "clipping/estrategiapuel.h"
#include <boost/test/unit_test.hpp>

#include "clipping/estrategianear.h"

BOOST_AUTO_TEST_SUITE( testes )

class Fixture {
public:
	Fixture() : minhaWindow( Coordenada(-6.0, -6.0), Coordenada(6.0, 6.0), Coordenada(0.0,0.0, -6.0) ) {};
	~Fixture(){};
protected:
	Window minhaWindow;
};

#define COORDENADA_CHECK_CLOSE(a, b) BOOST_CHECK( coordenada_check_close(a, b) )

boost::test_tools::predicate_result coordenada_check_close(Coordenada um, Coordenada dois) {
	if(  ( abs( um.x() - dois.x() ) > 0.0001) ||
		 ( abs( um.x() - dois.x() ) > 0.0001) ||
		 ( abs( um.x() - dois.x() ) > 0.0001)
	){
		boost::test_tools::predicate_result erro(false);
		erro.message() << um << " != " << dois;
		return erro;
	}
	return true;
};

BOOST_FIXTURE_TEST_SUITE( clipping, Fixture )

//TODO Testar clipping de reta
//TODO Testar clipping de ponto
//TODO Descomentar testes antigos
//TODO Converter os outros testes
BOOST_AUTO_TEST_CASE( ponto ) {
	;
}

BOOST_AUTO_TEST_CASE( clipparParaONear){
	std::vector<Coordenada> pontos;
	Window janela( Coordenada(-10.0, -10.0, 10.0), Coordenada(10.0,10.0,10.0), Coordenada(0.0,0.0,0.0));
	pontos.push_back(Coordenada(0.0, 0.0, 13.0));
	pontos.push_back(Coordenada(10.0, 0.0, 13.0));
	pontos.push_back(Coordenada(10.0,15.0, 4.0));
	pontos.push_back(Coordenada(0.0, 15.0, 4.0));
	pontos.push_back(Coordenada(0.0, 0.0, 13.0));

	EstrategiaNear estrategia;

	pontos = estrategia.clipar(pontos, &janela);

	BOOST_CHECK_EQUAL( janela.near(), 10);
	BOOST_CHECK_EQUAL( pontos.size(), 6);
	COORDENADA_CHECK_CLOSE( pontos[0], Coordenada(0.0,0.0,13.0));
	COORDENADA_CHECK_CLOSE( pontos[1], Coordenada(10.0,0.0,13.0));
	COORDENADA_CHECK_CLOSE( pontos[2], Coordenada(10.0,5.0,10.0));
	COORDENADA_CHECK_CLOSE( pontos[3], Coordenada());
	COORDENADA_CHECK_CLOSE( pontos[4], Coordenada(0.0,5.0,10.0));
}

BOOST_AUTO_TEST_CASE( poligonoDentroDaWindow ) {
	//	Coordenadas verticesQuadrado;
	//	verticesQuadrado.push_back(Coordenada(-5.0,-5.0));
	//	verticesQuadrado.push_back(Coordenada(5.0,-5.0));
	//	verticesQuadrado.push_back(Coordenada(5.0,5.0));
	//	verticesQuadrado.push_back(Coordenada(-5.0,5.0));
	//	Poligono quadrado("Sem nome", verticesQuadrado);
	//
	//	compararChar( clipador.obterRegiao(Coordenada(-5.0,-5.0), &minhaWindow), 0 );
	//	compararChar( clipador.obterRegiao(Coordenada(5.0,-5.0), &minhaWindow), 0 );
	//	compararChar( clipador.obterRegiao(Coordenada(5.0,5.0), &minhaWindow), 0 );
	//	compararChar( clipador.obterRegiao(Coordenada(-5.0,5.0), &minhaWindow), 0 );
	//
	//	Coordenadas verticesQuadradoClipado = quadrado.coordenadas(&minhaWindow);
	//
	//	QCOMPARE(verticesQuadradoClipado[0], Coordenada(-5.0,-5.0));
	//	QCOMPARE(verticesQuadradoClipado[1], Coordenada(5.0,-5.0));
	//	QCOMPARE(verticesQuadradoClipado[2], Coordenada(5.0,5.0));
	//	QCOMPARE(verticesQuadradoClipado[3], Coordenada(-5.0,5.0));
	//	QCOMPARE(verticesQuadradoClipado[4], Coordenada(-5.0,-5.0));
}

BOOST_AUTO_TEST_CASE( poligonoPraDireita ) {
	//	Coordenadas vertices;
	//	vertices.push_back(Coordenada(-5.0, -5.0));
	//	vertices.push_back(Coordenada(7.0, -5.0));
	//	vertices.push_back(Coordenada(7.0, 5.0));
	//	vertices.push_back(Coordenada(-5.0, 5.0));
	//
	//	compararChar( clipador.obterRegiao(Coordenada(-5.0,-5.0), &minhaWindow), 0 );
	//	compararChar( clipador.obterRegiao(Coordenada(7.0,-5.0), &minhaWindow), RIGHT );
	//	compararChar( clipador.obterRegiao(Coordenada(7.0,5.0), &minhaWindow), RIGHT );
	//	compararChar( clipador.obterRegiao(Coordenada(-5.0,5.0), &minhaWindow), 0 );
	//
	//	Poligono quadrado("Sem nome", vertices);
	//
	//	Coordenadas clipada = quadrado.coordenadas(&minhaWindow);
	//	QCOMPARE( clipada[0], Coordenada(-5.0, -5.0));
	//	QCOMPARE( clipada[1], Coordenada(6.0, -5.0));
	//	Q_ASSERT( clipada[2].isNull() );
	//	QCOMPARE( clipada[3], Coordenada(6.0, 5.0));
	//	QCOMPARE( clipada[4], Coordenada(-5.0, 5.0));
	//	QCOMPARE( clipada[5], Coordenada(-5.0, -5.0));
}

BOOST_AUTO_TEST_CASE( regiaoExclusiva ) {
	EstrategiaPuel estrategia;
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(-7.0,0.0), &minhaWindow), LEFT);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(0.0, 7.0), &minhaWindow), TOP);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(7.0, 0.0), &minhaWindow), RIGHT);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(0.0, -7.0), &minhaWindow), BOTTOM);

	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(-10.0,7.0), &minhaWindow), LEFT);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(-7.0,10.0), &minhaWindow), TOP);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(7.0,10.0), &minhaWindow), TOP);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(10.0,7.0), &minhaWindow), RIGHT);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(10.0,-7.0), &minhaWindow), RIGHT);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(7.0,-10.0), &minhaWindow), BOTTOM);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(-7.0,-10.0), &minhaWindow), BOTTOM);
	BOOST_CHECK_EQUAL(estrategia.regiaoExclusiva(Coordenada(-10.0,-7.0), &minhaWindow), LEFT);
}

BOOST_AUTO_TEST_SUITE_END()
BOOST_AUTO_TEST_SUITE_END()
