#include "matriz.h"
#include "coordenada.h"
#include <boost/test/unit_test.hpp>

BOOST_AUTO_TEST_SUITE( testes )
BOOST_AUTO_TEST_SUITE( matriz )

BOOST_AUTO_TEST_CASE( getSet ) {
	Matriz<2,2> ola;
	ola[0][0] = 1.0;
	ola[0][1] = 2.0;
	ola[1][0] = 3.0;
	ola[1][1] = 4.0;
	BOOST_CHECK_EQUAL(1.0, ola[0][0]);
	BOOST_CHECK_EQUAL(2.0, ola[0][1]);
	BOOST_CHECK_EQUAL(3.0, ola[1][0]);
	BOOST_CHECK_EQUAL(4.0, ola[1][1]);
	Matriz<2,2> copia = ola;
	BOOST_CHECK_EQUAL(ola[0][0], copia[0][0]);
	BOOST_CHECK_EQUAL(ola[0][1], copia[0][1]);
	BOOST_CHECK_EQUAL(ola[1][0], copia[1][0]);
	BOOST_CHECK_EQUAL(ola[1][1], copia[1][1]);
	copia *= copia;
	BOOST_CHECK_EQUAL(copia[0][0], 1.0*1.0 + 2.0*3.0);
	BOOST_CHECK_EQUAL(copia[0][1], 1.0*2.0 + 2.0*4.0);
	BOOST_CHECK_EQUAL(copia[1][0], 3.0*1.0 + 4.0*3.0);
	BOOST_CHECK_EQUAL(copia[1][1], 3.0*2.0 + 4.0*4.0);
}

BOOST_AUTO_TEST_CASE( matrizIdentidade ){
	Matriz<2,2> ola;
	Matriz<2,2> identidade;
	ola[0][0] = 1.0;
	ola[0][1] = 2.0;
	ola[1][0] = 3.0;
	ola[1][1] = 4.0;
	identidade[0][0] = 1.0;
	identidade[0][1] = 0.0;
	identidade[1][0] = 0.0;
	identidade[1][1] = 1.0;
	Matriz<2,2> resultado = ola*identidade;
	BOOST_CHECK_EQUAL(ola[0][0], resultado[0][0]);
	BOOST_CHECK_EQUAL(ola[0][1], resultado[0][1]);
	BOOST_CHECK_EQUAL(ola[1][0], resultado[1][0]);
	BOOST_CHECK_EQUAL(ola[1][1], resultado[1][1]);
	ola *= identidade;
	BOOST_CHECK_EQUAL(ola[0][0], resultado[0][0]);
	BOOST_CHECK_EQUAL(ola[0][1], resultado[0][1]);
	BOOST_CHECK_EQUAL(ola[1][0], resultado[1][0]);
	BOOST_CHECK_EQUAL(ola[1][1], resultado[1][1]);
}

BOOST_AUTO_TEST_CASE( matrizEscala ){
	Matriz<2,2> ola;
	Matriz<2,2> identidade;
	ola[0][0] = 1.0;
	ola[0][1] = 2.0;
	ola[1][0] = 3.0;
	ola[1][1] = 4.0;
	identidade[0][0] = 3.0;
	identidade[0][1] = 0.0;
	identidade[1][0] = 0.0;
	identidade[1][1] = 3.0;
	Matriz<2,2> resultado = ola*identidade;
	BOOST_CHECK_EQUAL(ola[0][0]*3, resultado[0][0]);
	BOOST_CHECK_EQUAL(ola[0][1]*3, resultado[0][1]);
	BOOST_CHECK_EQUAL(ola[1][0]*3, resultado[1][0]);
	BOOST_CHECK_EQUAL(ola[1][1]*3, resultado[1][1]);
	ola *= identidade;
	BOOST_CHECK_EQUAL(ola[0][0], resultado[0][0]);
	BOOST_CHECK_EQUAL(ola[0][1], resultado[0][1]);
	BOOST_CHECK_EQUAL(ola[1][0], resultado[1][0]);
	BOOST_CHECK_EQUAL(ola[1][1], resultado[1][1]);
}

BOOST_AUTO_TEST_CASE(testesMultiplicacaoQualquerComVetor) {
	Matriz<1,3> vetor;
	Matriz<3,2> multiplicador;
	Matriz<1,2> esperado;
	Matriz<1,2> resultado;
	vetor[0][0] = 1;
	vetor[0][1] = 2;
	vetor[0][2] = 3;
	multiplicador[0][0] = 7;
	multiplicador[1][0] = 8;
	multiplicador[2][0] = 9;
	multiplicador[0][1] = 10;
	multiplicador[1][1] = 11;
	multiplicador[2][1] = 12;
	esperado[0][0] = 1*7 + 2*8 + 3*9;
	esperado[0][1] = 1*10 + 2*11 + 3*12;
	resultado = vetor*multiplicador;
	BOOST_CHECK_EQUAL(esperado[0][0], resultado[0][0]);
	BOOST_CHECK_EQUAL(esperado[0][1], resultado[0][1]);
}

BOOST_AUTO_TEST_CASE(testeCoordenadaNula) {
	Coordenada nula;
	BOOST_CHECK(nula.isNull());
	Coordenada outra(nula);
	BOOST_CHECK(outra.isNull());
	Coordenada oscilante(0.0, 0.0);
	BOOST_CHECK(!oscilante.isNull());
	oscilante = nula;
	BOOST_CHECK(oscilante.isNull());
	oscilante = Coordenada(2.0, 3.0);
	BOOST_CHECK(!oscilante.isNull());

	oscilante = nula;
	BOOST_CHECK(oscilante.isNull());

	oscilante.x( 2.0 );
	oscilante.y( 3.0 );
	oscilante.z( 4.0 );
	BOOST_CHECK(!oscilante.isNull());

}

BOOST_AUTO_TEST_SUITE_END()
BOOST_AUTO_TEST_SUITE_END()
