#include "vetor.h"
#include "transformacao.h"

#include <boost/test/unit_test.hpp>

void vetorClose(Vetor um, Vetor dois) {
	BOOST_CHECK_CLOSE( um.x(), dois.x(), 0.000001 );
	BOOST_CHECK_CLOSE( um.y(), dois.y(), 0.000001 );
	BOOST_CHECK_CLOSE( um.z(), dois.z(), 0.000001 );
}

#define REQUIRE_CLOSE(a, b) BOOST_REQUIRE( requer_close(a, b) )

boost::test_tools::predicate_result requer_close(Vetor um, Vetor dois) {
	if(  ( abs( um.x() - dois.x() ) > 0.0001) ||
		 ( abs( um.x() - dois.x() ) > 0.0001) ||
		 ( abs( um.x() - dois.x() ) > 0.0001)
	){
		boost::test_tools::predicate_result erro(false);
		erro.message() << um << " != " << dois;
		return erro;
	}
	return true;
};

BOOST_AUTO_TEST_SUITE( testes )
BOOST_AUTO_TEST_SUITE( vetor )

BOOST_AUTO_TEST_CASE( testeModulo ){
	Vetor praCima(0.0, 10.0);
	Vetor pitagoras(3.0,4.0);
	BOOST_CHECK_EQUAL(praCima.y(), praCima.modulo());
	BOOST_CHECK_EQUAL(5.0, pitagoras.modulo());
}

BOOST_AUTO_TEST_CASE( testeModulo3D ) {
	Vetor outraDimensao(0.0, 0.0, 10.0);
	Vetor pitagoras3D(3.0, 0.0, 4.0);
	Vetor aleatoriaDoWolfram(6.0, 6.0, 6.0);
	BOOST_CHECK_EQUAL(outraDimensao.z(), outraDimensao.modulo());
	BOOST_CHECK_EQUAL(pitagoras3D.modulo(), 5.0);
	BOOST_CHECK_CLOSE(aleatoriaDoWolfram.modulo(),
			10.392304845413263761164678049035234201656831522862283768334841876711598101452800222486877120543491454,
			0.000000001);
}

BOOST_AUTO_TEST_CASE( transformacaoDeVetor ) {
	Vetor um(1.0, 2.0, 3.0);
	Transformacao mover = Transformacao::translacao(1.0, 2.0, 3.0);
	vetorClose( um*mover, Vetor(2.0, 4.0, 6.0) );
	Transformacao escalar = Transformacao::escala(2.0, 3.0, 4.0);
	vetorClose( um*escalar, Vetor(2.0, 6.0, 12.0));
}

BOOST_AUTO_TEST_CASE( crossProduct ){
	BOOST_CHECK_EQUAL(Vetor(1.0,2.0,4.0).crossProduct(Vetor(4.0,6.0,8.0)), Vetor(-8.0,8.0,-2.0));

	BOOST_CHECK_EQUAL(Vetor(6.0,6.0,6.0).crossProduct(Vetor(2.0,3.0,1.0)), Vetor(-12.0,6.0,6.0));

	BOOST_CHECK_EQUAL(Vetor(1.0,1.0,1.0).crossProduct(Vetor(-1.0,1.0,-1.0)), Vetor(-2.0,0.0,2.0));
}

BOOST_AUTO_TEST_CASE( testeAngulo3D ){
	Vetor praCima(0.0, 1.0, 0.0);
	Vetor proLado(1.0, 0.0, 0.0);
	Vetor proCanto(1.0, 1.0, 0.0);
	Vetor praBaixo(0.0, -1.0, 0.0);
	Vetor proCantoDeBaixoDaEsquerda(-1.0, -1.0, 0.0);
	BOOST_CHECK_EQUAL(praCima.angulo(proLado).z(), 90.0);
	BOOST_CHECK_CLOSE(praCima.angulo(proCanto).z(), 45.0, 0.0000000001);
	BOOST_CHECK_EQUAL(praCima.angulo(praBaixo).z(), 180.0);
	BOOST_CHECK_EQUAL(praCima.angulo(proCantoDeBaixoDaEsquerda).z(), 225.0);

	Vetor praFrente(0.0,0.0,1.0);
	BOOST_CHECK_EQUAL(praCima.angulo(praFrente),  Vetor(90.0, 0.0, 0.0) );
	BOOST_CHECK_EQUAL(praFrente.angulo(praCima), Vetor(360.0-90.0, 0.0, 0.0));
	BOOST_CHECK_EQUAL(praFrente.angulo(proLado), Vetor(0.0, 90.0, 0.0));
	BOOST_CHECK_EQUAL(proLado.angulo(praFrente), Vetor(0.0, 360-90.0, 0.0));

	BOOST_CHECK_EQUAL(proCantoDeBaixoDaEsquerda.angulo(praFrente), Vetor(270.0,90.0, 0.0));
	BOOST_CHECK_EQUAL(praFrente.angulo(proCantoDeBaixoDaEsquerda), Vetor(90.0,270.0, 0.0));
/*
	Vetor praTras(0.0, 0.0, -1.0);
	for(double x=0; x < 360.0; x += 5.0)
	for(double y=0; y < 360.0; y += 5.0)
	for(double z=0; z < 360.0; z += 5.0){
		Transformacao rotacao = Transformacao::rotacao(x, y, z);
		Vetor novoPraCima = praCima;
		Vetor novoProLado = proLado;
		Vetor novoPraBaixo = praBaixo;
		novoPraCima *= rotacao;
		novoProLado *= rotacao;
		novoPraBaixo *= rotacao;

		REQUIRE_CLOSE( praCima.angulo( novoPraCima ), Vetor(x, y, z) );
		REQUIRE_CLOSE( proLado.angulo( novoProLado ), Vetor(x, y, z) );
		REQUIRE_CLOSE( praBaixo.angulo( novoPraBaixo ), Vetor(x, y, z) );

	};*/
}

BOOST_AUTO_TEST_CASE( testeMultiplicacaoPorEscalar ){
	Coordenada pontoDoCima(0.0, 20.0);
	Vetor cima(pontoDoCima);
	Vetor lado(20.0, 0.0);

	BOOST_CHECK_EQUAL(pontoDoCima*0.1, Coordenada(0.0, 2.0));
	BOOST_CHECK_EQUAL(lado*0.1, Vetor(2.0, 0.0));
	BOOST_CHECK_EQUAL(cima*0.1, Vetor(0.0, 2.0));
}

BOOST_AUTO_TEST_CASE( testeMultiplicacaoPorEscalar3D ){
	Vetor um(0.0, 20.0, 30.0);
	Vetor outro(10.0, 5.0, 2.5);
	BOOST_CHECK_EQUAL( um*0, Vetor(0.0,0.0,0.0) );
	BOOST_CHECK_EQUAL( outro*1.0, outro );
	BOOST_CHECK_EQUAL( outro*0.5, Vetor(5.0, 2.5, 1.25) );
	BOOST_CHECK_EQUAL( um*2.0, Vetor(0.0, 40.0, 60.0) );
}

BOOST_AUTO_TEST_CASE( testeMultiplicacaoUmAUm ) {
	Vetor um(1.0, 2.0, 3.0);
	Vetor dois(3.0, 4.0, 5.0);
	Vetor tres(0.0, 1.0, 1.0);
	Vetor quatro(1.0, 0.0, 1.0);
	Vetor cinco(0.0, 0.0, 1.0);

	BOOST_CHECK_EQUAL( um*dois, Vetor(3.0, 8.0, 15.0) );
	BOOST_CHECK_EQUAL( um*tres, Vetor(0.0, 2.0, 3.0) );
	BOOST_CHECK_EQUAL( tres*cinco, Vetor(0.0, 0.0, 1.0) );
}

BOOST_AUTO_TEST_CASE( testeProdutoEscalar3D ) {
	Vetor um(1.0, 2.0, 3.0);
	Vetor dois(2.0,3.0,4.0);
	Vetor tres(3.0,4.0,5.0);
	BOOST_CHECK_EQUAL( um.produtoEscalar(dois), 1.0*2.0 + 2.0*3.0 + 3.0*4.0);
	BOOST_CHECK_EQUAL( dois.produtoEscalar(tres), 2.0*3.0 + 3.0*4.0 + 4.0*5.0);
}

BOOST_AUTO_TEST_CASE( testeNormalized ) {
	Vetor qualquerUm(1.0, 2.0, 3.0);
	Vetor demonio(6.0, 6.0, 6.0);
	Vetor qualquerOutro(2.0, 3.0, 16.0);
	vetorClose( qualquerOutro.normalized(), Vetor(2.0/sqrt(269.0), 3.0/sqrt(269.0), 16.0/sqrt(269.0)) );
	vetorClose( qualquerUm.normalized(), Vetor(1.0/sqrt(14.0), 2.0/sqrt(14.0), 3.0/sqrt(14.0)) );
	vetorClose( demonio.normalized(), Vetor(1.0/sqrt(3.0), 1.0/sqrt(3.0), 1.0/sqrt(3.0)) );
}

//TODO Angulo 3D entre vetores.

BOOST_AUTO_TEST_SUITE_END()
BOOST_AUTO_TEST_SUITE_END()
