#include "window.h"
#include "ponto.h"
#include <boost/test/unit_test.hpp>
#include <vector>

class Fixture{
public:
	Fixture(): janela(Coordenada(0.0,0.0), Coordenada(10.0,10.0), Coordenada(0.0,0.0,-10.0)),
			outra(Coordenada(-10.0,-10,0.0), Coordenada(10.0, 10.0, 0.0), Coordenada(0.0,0.0,-10.0))
	{
		pontinho = new Ponto("pontinho", Coordenada(3.0, 3.0, 13.0));
		pontinho2 = new Ponto("pontinho2", Coordenada(3.0, 3.0, 3.0));
		displayFile.push_back(pontinho);
		displayFile.push_back(pontinho2);
		outra.atualizarCPP(displayFile);

		BOOST_REQUIRE_EQUAL( outra.centro(), Coordenada(0.0,0.0,0.0));
		BOOST_REQUIRE_EQUAL( outra.coordenadasDePlanoDeProjecao().size(), 2 );
//		BOOST_REQUIRE_EQUAL(*(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()), Coordenada(3.0, 3.0, 3.0));
		BOOST_REQUIRE_EQUAL( outra.viewPlaneNormal().modulo(), 20.0 );
	}

	~Fixture(){
		delete pontinho;
		delete pontinho2;
	}
	void vetorClose(Vetor um, Vetor dois) {
		BOOST_CHECK_CLOSE( um.x(), dois.x(), 0.00001 );
		BOOST_CHECK_CLOSE( um.y(), dois.y(), 0.00001 );
		BOOST_CHECK_CLOSE( um.z(), dois.z(), 0.00001 );
	}
	void rodar(double x, double y, double z) {
		outra.rodar(x, y, z);
		Coordenada anterior = *(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin());
		outra.atualizarCPP(displayFile);
		Coordenada atual = *(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin());
		vetorClose(anterior, atual);
	}

protected:
	Window janela, outra;
	Ponto *pontinho, *pontinho2;
	DisplayFile displayFile;
};

BOOST_AUTO_TEST_SUITE( testes )
BOOST_FIXTURE_TEST_SUITE( window, Fixture )

BOOST_AUTO_TEST_CASE( testeTranslacao ){
	Transformacao translacao = Transformacao::translacao(10,5);
	janela.aplicarTransformacao(translacao);

	Coordenada inferior = janela.inferiorEsquerdo();
	Coordenada superior = janela.superiorDireito();

	BOOST_CHECK_EQUAL(inferior.x(), 10.0);
	BOOST_CHECK_EQUAL(inferior.y(), 5.0);

	BOOST_CHECK_EQUAL(superior.x(), 20.0);
	BOOST_CHECK_EQUAL(superior.y(), 15.0);
}

BOOST_AUTO_TEST_CASE( viewPlaneNormal ) {
	BOOST_CHECK_EQUAL( janela.viewPlaneNormal().normalized(), Vetor(0.0, 0.0, 1.0) );
}

BOOST_AUTO_TEST_CASE( tamanhoDaViewPlaneNormal ) {
	BOOST_CHECK_EQUAL( janela.viewPlaneNormal().modulo(), (janela.viewRightVector().modulo() + janela.viewUpVector().modulo())/2.0  );
}

//BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoX ) {
//	outra.rodar(90.0, 0.0, 0.0);
//	BOOST_CHECK_EQUAL(*(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()), Coordenada(3.0, 3.0, -3.0));
//}

//BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoY ) {

//	outra.rodar(0.0, 90.0, 0.0);
//	BOOST_CHECK_EQUAL(*(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()), Coordenada(-3.0, 3.0, 3.0));
//}

/*BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoZ ) {
	outra.rodar(0.0, 0.0, 90.0);
	BOOST_CHECK_EQUAL(*(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()), Coordenada(3.0, -3.0, 3.0));
}*/

/*BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoZAtualizandoCpp ) {
	outra.rodar(0.0, 0.0, 90.0);
	outra.atualizarCPP(displayFile);
	vetorClose(*(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()), Coordenada(3.0, -3.0, 3.0));
}

BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoYAtualizandoCpp ) {
	outra.rodar(0.0, 90.0, 0.0);
	outra.atualizarCPP(displayFile);
	vetorClose(*(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()), Coordenada(-3.0, 3.0, 3.0));
};

BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoXAtualizandoCpp ) {
	outra.rodar(90.0, 0.0, 0.0);
	outra.atualizarCPP(displayFile);
	vetorClose(*(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()), Coordenada(3.0, 3.0, -3.0));
}*/

/*BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoX45 ) {
	outra.rodar(45.0, 0.0, 0.0);
	BOOST_CHECK_CLOSE(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()->z()+1.0, 1.0, 0.001);
	BOOST_CHECK_EQUAL(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()->x(), 3.0);
}

BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoY45 ) {
	outra.rodar(0.0, 45.0, 0.0);
	BOOST_CHECK_CLOSE( outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()->x()+1.0, 1.0, 0.001);
	BOOST_CHECK_EQUAL(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()->y(), 3.0);
}

BOOST_AUTO_TEST_CASE( compensacaoRotacaoEixoZ45 ) {
	outra.rodar(0.0, 0.0, 45.0);
	BOOST_CHECK_CLOSE(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()->y()+1.0, 1.0, 0.001);
	BOOST_CHECK_EQUAL(outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin()->z(), 3.0);
}

BOOST_AUTO_TEST_CASE( compensacaoRotacaoDepoisDoCppVariosEixos ) {
	rodar(23.0, 0.0, 0.0);
//	rodar(0.0, 43.0, 0.0);
}*/

BOOST_AUTO_TEST_CASE( testeEscalonamento ){

	Transformacao escala = Transformacao::escala(2.0,2.0,2.0);
	janela.aplicarTransformacao(escala);

	Coordenada inferior = janela.inferiorEsquerdo();
	Coordenada superior = janela.superiorDireito();

	BOOST_CHECK_EQUAL(inferior.x(), 0.0);
	BOOST_CHECK_EQUAL(inferior.y(), 0.0);
	BOOST_CHECK_EQUAL(inferior.z(), 0.0);

	BOOST_CHECK_EQUAL(superior.x(), 20.0);
	BOOST_CHECK_EQUAL(superior.y(), 20.0);
	BOOST_CHECK_EQUAL(superior.z(), 0.0);
}

/*BOOST_AUTO_TEST_CASE( testeObjetosDaWindowMovida ) {
	outra.mover(0.0, 0.0, -1.0);

	BOOST_CHECK_EQUAL( *outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin(), Coordenada(3.0, 3.0, 23.0) );
	outra.mover(0.0, 0.0, 2.0);
	BOOST_CHECK_EQUAL( *outra.coordenadasDePlanoDeProjecao()[0]->coordenadasBegin(), Coordenada(3.0, 3.0, -17.0) );

}

BOOST_AUTO_TEST_CASE( testeMoverWindowTerceiraDimensao ){
	std::vector<Coordenada> antigas(outra.coordenadasBegin(), outra.coordenadasEnd());

	outra.mover(0.0, 0.0, 1.0);

	BOOST_REQUIRE_EQUAL( antigas.size(), outra.coordenadasSize() );
	Window::iterator i = antigas.begin();
	for(Window::iterator j = outra.coordenadasBegin(); j != outra.coordenadasEnd(); ++j, ++i){
		BOOST_CHECK_EQUAL( *j, *i+Coordenada(0.0,0.0, 20.0 ) );
	}

}

BOOST_AUTO_TEST_CASE( testeCopNaOrigem ) {
	BOOST_CHECK_EQUAL(outra.coordenadasDePlanoDeProjecao()[1]->coordenadasBegin()->z(), -7.0);
}

BOOST_AUTO_TEST_CASE( testeTransladandoCop ) {
	outra.mover(0.0, 0.0, 1.0);
	BOOST_CHECK_EQUAL(outra.centerOfProjection(), Coordenada(0.0, 0.0, 10.0));
	outra.mover(0.0, 1.0, 1.0);
	BOOST_CHECK_EQUAL(outra.centerOfProjection(), Coordenada(0.0, 20.0, 30.0));
	outra.mover(-1.0, 0.0, -1.0);
	BOOST_CHECK_EQUAL(outra.centerOfProjection(), Coordenada(-20.0, 20.0, 10.0));
}*/

BOOST_AUTO_TEST_SUITE_END()
BOOST_AUTO_TEST_SUITE_END()
