#include "vetor.h"
#include <cmath>

Vetor::Vetor(double x, double y, double z):Coordenada(x,y,z) {}
Vetor::Vetor(const Matriz<1,COORDENADA_LARGURA>& matriz):Coordenada(matriz){}
Vetor::Vetor(const Coordenada& coordenada):Coordenada(coordenada){}
Vetor::~Vetor(){
}

Vetor Vetor::operator*(const Transformacao& transformacao) const{
	Coordenada fim = *this;
	fim *= transformacao;
	return fim;
}

double Vetor::produtoEscalar(const Vetor& that) const{
	Vetor produtoUmAUm = (*this)*that;
	return produtoUmAUm.x() + produtoUmAUm.y() + produtoUmAUm.z();
}

double Vetor::modulo() const{
	return sqrt(  pow(x(),2) + pow(y(),2) + pow(z(), 2) );
}
#include <iostream>
Vetor Vetor::angulo(const Vetor& that) const{
	//TODO Refactor

	Vetor thisX = (*this)	*	Vetor(0.0, 1.0, 1.0);
	Vetor thatX = that		*	Vetor(0.0, 1.0, 1.0);
	Vetor thisY = (*this)	*	Vetor(1.0, 0.0, 1.0);
	Vetor thatY = that		*	Vetor(1.0, 0.0, 1.0);
	Vetor thisZ = (*this)	*	Vetor(1.0, 1.0, 0.0);
	Vetor thatZ = that		*	Vetor(1.0, 1.0, 0.0);

	double radianosX = acos(thisX.produtoEscalar(thatX) / (thisX.modulo()*thatX.modulo()) );
	double radianosY = acos(thisY.produtoEscalar(thatY) / (thisY.modulo()*thatY.modulo()) );
	double radianosZ = acos(thisZ.produtoEscalar(thatZ) / (thisZ.modulo()*thatZ.modulo()) );

	if( radianosX != radianosX)
		radianosX = 0.0;
	if( radianosY != radianosY)
		radianosY = 0.0;
	if( radianosZ != radianosZ)
		radianosZ = 0.0;

	Vetor giradoZ = thisZ.girado90GrausEmZ();
	if( thatZ.produtoEscalar(giradoZ)/pow(giradoZ.modulo(),2) < 0 )
		radianosZ = 2*M_PI - radianosZ;

	Vetor giradoX = thisX.girado90GrausEmX();
	if( thatX.produtoEscalar(giradoX)/pow(giradoX.modulo(),2) < 0 )
		radianosX = 2*M_PI - radianosX;

	Vetor giradoY = thisY.girado90GrausEmY();
	if( thatY.produtoEscalar(giradoY)/pow(giradoY.modulo(),2) < 0 )
		radianosY = 2*M_PI - radianosY;

	return Vetor(radianosX*180/M_PI, radianosY*180/M_PI, radianosZ*180/M_PI);
}

Vetor Vetor::normalized() const{
	return (*this) * ( 1.0/this->modulo() );
}

Vetor Vetor::operator*(const Vetor& that) const{
	return Vetor( this->x()*that.x(), this->y()*that.y(), this->z()*that.z() );
}

Vetor Vetor::operator*(double escalar) const{
	return (*this) * Vetor(escalar, escalar, escalar);
}

Vetor Vetor::girado90GrausEmX() const{
	return Vetor(x(), -z(), y());
}
Vetor Vetor::girado90GrausEmY() const{
	return Vetor(z(), y(), -x());
}
Vetor Vetor::girado90GrausEmZ() const{
	return Vetor(y(), -x(), z());
}

Vetor Vetor::crossProduct(const Vetor &outro) const{
    Matriz<3, 3> base;
    base[0][1] = -z();	base[0][2] = y();
    base[1][0] = z();	base[1][2] = -x();
    base[2][0] = -y();	base[2][1] = x();
    Matriz<3,1> transposta;
    transposta[0][0] = outro.x();
    transposta[1][0] = outro.y();
    transposta[2][0] = outro.z();
    transposta = base*transposta;
    return Vetor(transposta[0][0], transposta[1][0], transposta[2][0]);
}

std::ostream& operator<<(std::ostream& stream, Vetor escopo){
//	stream << "Vetor(";
//	stream << escopo.x() << ", " << escopo.y() << ", " << escopo.z();
//	stream << ")";

	stream << "0 0 0 " << std::fixed << escopo.x() << ' ' << escopo.y() << ' ' << escopo.z();
	return stream;
}
