#include "window.h"

Window::Window(Coordenada inferiorEsquerdo, Coordenada superiorDireito, Coordenada centroDaProjecao) : Objeto("window", Cor(0.0,0.0,0.0))
{
	_centerOfProjection = centroDaProjecao;
	_coordenadas.push_back(inferiorEsquerdo);
	_coordenadas.push_back( Coordenada(inferiorEsquerdo.x(), superiorDireito.y(), inferiorEsquerdo.z()) );
	_coordenadas.push_back(superiorDireito);
	_coordenadas.push_back( Coordenada(superiorDireito.x(), inferiorEsquerdo.y(), superiorDireito.z()) );
}

Window::~Window(){

}

void Window::persist(TiXmlNode * where) const {
	TiXmlElement* cop = new TiXmlElement("cop");
	TiXmlElement* vpn = new TiXmlElement("vpn");
	TiXmlElement* vup = new TiXmlElement("vup");
	TiXmlElement* wmin = new TiXmlElement("wmin");
	TiXmlElement* wmax = new TiXmlElement("wmax");
	TiXmlElement* compensacao = new TiXmlElement("compensacao");

	_centerOfProjection.persist(cop);
	viewPlaneNormal().persist(vpn);
	viewUpVector().persist(vup);
	_coordenadas.at(0).persist(wmin);
	_coordenadas.at(2).persist(wmax);
	this->compensacao.persist(compensacao);

	where->LinkEndChild(cop);
	where->LinkEndChild(vpn);
	where->LinkEndChild(vup);
	where->LinkEndChild(wmin);
	where->LinkEndChild(wmax);
	where->LinkEndChild(compensacao);

}
#include <iostream>
void Window::restore(TiXmlNode * where){
	compensacao = Transformacao();

	TiXmlNode* cop = where->FirstChild("cop");
	TiXmlNode* vup = where->FirstChild("vup");
	TiXmlNode* wmin = where->FirstChild("wmin");
	TiXmlNode* wmax = where->FirstChild("wmax");
	TiXmlNode* compensacaoNode = where->FirstChild("compensacao");

	if(cop == NULL || vup == NULL || wmin == NULL || wmax == NULL)
		throw std::string("Cop, vup, wmin ou wmax não especificados no XML");

	if(compensacaoNode == NULL){
		std::cerr << "Compensação não definida no XML, a window irá para posição inicial!" << std::endl;
		return;
	}

	_centerOfProjection.restore(cop->ToElement());
	_coordenadas[0].restore(wmin->ToElement());
	Vetor viewUp;
	viewUp.restore(vup->ToElement());
	_coordenadas[1] = _coordenadas[0] + viewUp;
	_coordenadas[2].restore(wmax->ToElement());

	this->compensacao.restore(compensacaoNode->ToElement());

};


Coordenada Window::inferiorEsquerdo() const{
	return _coordenadas[0];
}
void Window::inferiorEsquerdo(const Coordenada& nova){
	_coordenadas[0] = nova;
}

Coordenada Window::superiorDireito() const {
	return _coordenadas[2];
}
void Window::superiorDireito(const Coordenada& nova){
	_coordenadas[2] = nova;
}

void Window::mover(double sentidoX, double sentidoY, double sentidoZ){

	Vetor direcao = (viewUpVector()*sentidoY)+(viewRightVector()*sentidoX)+(viewPlaneNormal()*sentidoZ);
	Vetor direcaoCompensacao = Coordenada(viewRightVector().modulo()*-sentidoX,
							   viewUpVector().modulo()*-sentidoY,
							   viewPlaneNormal().modulo()*-sentidoZ);

	Transformacao movimentoDaWindow = Transformacao::translacao(direcao.x(), direcao.y(), direcao.z());
	this->aplicarTransformacao(
			movimentoDaWindow
		);
	_centerOfProjection *= movimentoDaWindow;

	Transformacao destranslacao = Transformacao::translacao(direcaoCompensacao.x(), direcaoCompensacao.y(), direcaoCompensacao.z());
	coordenadasDePlanoDeProjecao().aplicarTransformacao(destranslacao);

	compensacao *= destranslacao;
}

void Window::escala(double proporcao){
	double tamanhoMinimo = 5.0;
	if( (this->viewUpVector().modulo() > tamanhoMinimo && this->viewRightVector().modulo() > tamanhoMinimo) ||
		proporcao > 1.0
		){
		Transformacao escala = Transformacao::escala(proporcao, proporcao, proporcao, centro().x(), centro().y(), centro().z());
		this->aplicarTransformacao(escala);
	}

}
#include <iostream>
void Window::atualizarCPP(const DisplayFile& display) {
	_coordenadasDePlanoDeProjecao = display.clonarObjetos();

	coordenadasDePlanoDeProjecao().aplicarTransformacao(compensacao);
}

void Window::rodar(double roll, double pitch, double yaw) {
	Transformacao rotacao =
		Transformacao::rotacao(roll, pitch, yaw, centerOfProjection());
	aplicarTransformacao(rotacao );


	Transformacao desrotacao = Transformacao::rotacao(0.0, 0.0, -yaw) *
							   Transformacao::rotacao(0.0, -pitch, 0.0) *
							   Transformacao::rotacao(-roll, 0.0, 0.0);
	coordenadasDePlanoDeProjecao().aplicarTransformacao(desrotacao);

	compensacao *= desrotacao;
}

Coordenada Window::superiorEsquerdo() const {
	return _coordenadas[1];
}

void Window::superiorEsquerdo(const Coordenada& nova) {
	_coordenadas[1] = nova;
}

Vetor Window::viewUpVector() const {
	return _coordenadas[1] - _coordenadas[0];
}

Vetor Window::viewRightVector() const {
	return _coordenadas[3]- _coordenadas[0];
}

DisplayFile Window::coordenadasDePlanoDeProjecao() const {
	return _coordenadasDePlanoDeProjecao;
}

Vetor Window::viewPlaneNormal() const {
	Vetor direcao = viewRightVector().crossProduct(viewUpVector());
	double tamanho = ( viewRightVector().modulo() + viewUpVector().modulo() )/2.0;
	return direcao.normalized()*tamanho;
}

Coordenada Window::centerOfProjection() const {
	return _centerOfProjection;
}

double Window::near() const {
	return Vetor( _centerOfProjection - centro() ).modulo();
}
