#include "novoobjeto_options.h"
#include "ui_novoobjeto.h"
#include <QLabel>
#include <QLayout>

#include <QPushButton>

ColorOption::ColorOption(Type* gerenciado) {
	_gerenciado = gerenciado;
	QObject::connect(this, SIGNAL(colorChanged(QColor)), this, SLOT(valueChanged(QColor)));
}
void ColorOption::valueChanged(QColor nova) {
	((FabricaDeObjeto::Color*)_gerenciado)->value = Cor(nova);
	emit valueChanged();
}

BooleanOption::BooleanOption(Type *gerenciado) {
	_gerenciado = gerenciado;
	QObject::connect(this, SIGNAL(toggled(bool)), this, SLOT(valueChanged(bool)));
}
void BooleanOption::valueChanged(bool newValue) {
	((FabricaDeObjeto::Boolean*)_gerenciado)->value = newValue;
}

StringOption::StringOption(Type *gerenciado) {
	_gerenciado = gerenciado;
	QObject::connect(this, SIGNAL(textChanged(QString)), this, SLOT(valueChanged(QString)));
}
void StringOption::valueChanged(QString nova) {
	((FabricaDeObjeto::String*)_gerenciado)->value = nova.toStdString();
	emit valueChanged();
}

IntIntOption::IntIntOption(int pos) : _pos(pos) {
	origin = new QSpinBox();
	dest = new QSpinBox();

	origin->setMinimum(0);
	origin->setMaximum(99999);
	dest->setMinimum(0);
	dest->setMaximum(99999);

	QObject::connect(origin, SIGNAL(valueChanged(int)), this, SLOT(intIntChanged()));
	QObject::connect(dest, SIGNAL(valueChanged(int)), this, SLOT(intIntChanged()));

	QHBoxLayout* mainLayout = new QHBoxLayout(this);

	mainLayout->addWidget(origin);
	mainLayout->addWidget(dest);
}
void IntIntOption::intIntChanged() {
	emit intIntChanged(origin->value(), dest->value(), _pos);
}

CoordenadaOption::CoordenadaOption(int pos) : _pos(pos) {
	spinX = new QDoubleSpinBox();
	spinY = new QDoubleSpinBox();
	spinZ = new QDoubleSpinBox();

	spinX->setMinimum(-9999.9999);
	spinX->setMaximum(9999.9999);
	spinY->setMinimum(-9999.9999);
	spinY->setMaximum(9999.9999);
	spinZ->setMinimum(-9999.9999);
	spinZ->setMaximum(9999.9999);

	QObject::connect(spinX, SIGNAL(valueChanged(double)), this, SLOT(coordenadaChanged()));
	QObject::connect(spinY, SIGNAL(valueChanged(double)), this, SLOT(coordenadaChanged()));
	QObject::connect(spinZ, SIGNAL(valueChanged(double)), this, SLOT(coordenadaChanged()));

	QHBoxLayout* mainLayout = new QHBoxLayout(this);

	QLabel* posicao = new QLabel();
	posicao->setText(QString::number(_pos) + QString(": "));

	mainLayout->addWidget(posicao);
	mainLayout->addWidget(spinX);
	mainLayout->addWidget(spinY);
	mainLayout->addWidget(spinZ);
}
void CoordenadaOption::coordenadaChanged() {
	emit coordenadaChanged(spinX->value(), spinY->value(), spinZ->value(), _pos);
}

IntIntsOption::IntIntsOption(Type *gerenciado) {
	_gerenciado = gerenciado;
	QGridLayout* principal = new QGridLayout(this);
	add = new QPushButton();
	remove = new QPushButton();
	add->setText("+");
	remove->setText("-");
	principal->addWidget(add, 0, 0);
	principal->addWidget(remove, 0, 1);

	intIntsScrollArea = new QScrollArea();
	container = new QWidget();
	_intInts = new QVBoxLayout(container);
	_intInts->setSizeConstraint(QLayout::SetMinimumSize);
	container->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
	intIntsScrollArea->setWidget(container);

	QObject::connect(add, SIGNAL(released()), this, SLOT(addIntInt()));
	QObject::connect(remove, SIGNAL(released()), this, SLOT(removeIntInt()));

	remove->setEnabled(false);
	principal->addWidget(intIntsScrollArea, 1, 0, 1, 2);
}
void IntIntsOption::addIntInt() {
	IntIntOption* novo = new IntIntOption(_intInts->count());
	QObject::connect(novo, SIGNAL(intIntChanged(int,int,int)), this, SLOT(intIntChanged(int,int,int)));

	((FabricaDeObjeto::IntInt*)_gerenciado)->value.push_back(std::pair<int,int>(0,0));

	container->setGeometry(0,0,0,0);
	_intInts->addWidget(novo);

	remove->setEnabled(_intInts->count() > 0);
}
void IntIntsOption::removeIntInt() {
	QLayoutItem* toRemove = _intInts->itemAt(_intInts->count()-1);
	if(toRemove == NULL)
		return;

	container->setGeometry(0,0,0,0);
	toRemove->widget()->setParent(NULL);
	_intInts->removeItem(toRemove);
	container->setGeometry(0,0,0,0);

	remove->setEnabled(_intInts->count() > 0);
}
void IntIntsOption::intIntChanged(int origin, int dest, int pos) {
	((FabricaDeObjeto::IntInt*)_gerenciado)->value[pos] = std::pair<int,int>(origin,dest);
}

CoordenadasOption::CoordenadasOption(Type* gerenciado) {
	_gerenciado = gerenciado;
	QGridLayout* principal = new QGridLayout(this);
	add = new QPushButton();
	remove = new QPushButton();
	add->setText(QString("+"));
	remove->setText(QString("-"));
	principal->addWidget(add, 0, 0);
	principal->addWidget(remove, 0, 1);

	pointsScrollArea = new QScrollArea();
	container = new QWidget();
	_points = new QVBoxLayout(container);
	_points->setSizeConstraint(QLayout::SetMinimumSize);
	container->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
	pointsScrollArea->setWidget(container);

	QObject::connect(add, SIGNAL(released()), this, SLOT(addCoordenada()));
	QObject::connect(remove, SIGNAL(released()), this, SLOT(removeCoordenada()));

	remove->setEnabled(false);
	principal->addWidget(pointsScrollArea, 1, 0, 1, 2);
};
void CoordenadasOption::addCoordenada() {
	CoordenadaOption* novo = new CoordenadaOption(_points->count());
	QObject::connect(novo, SIGNAL(coordenadaChanged(double,double,double,int)), this, SLOT(coordenadaChanged(double,double,double,int)));

	((FabricaDeObjeto::Coordenadas*)_gerenciado)->value.push_back(Coordenada(0.0,0.0,0.0));
	int enoughQuantity = ((FabricaDeObjeto::Coordenadas*)_gerenciado)->enoughQuantity();

	container->setGeometry(0,0,0,0);
	_points->addWidget(novo);

	remove->setEnabled(_points->count() > 0);
	add->setEnabled( enoughQuantity < 1 );
	emit valueChanged();
}
void CoordenadasOption::removeCoordenada() {
	QLayoutItem* toRemove = _points->itemAt(_points->count()-1);
	if(toRemove == NULL)
		return;

	((FabricaDeObjeto::Coordenadas*)_gerenciado)->value.pop_back();

	container->setGeometry(0,0,0,0);
	toRemove->widget()->setParent(NULL);
	_points->removeItem(toRemove);
	container->setGeometry(0,0,0,0);

	int enoughQuantity = ((FabricaDeObjeto::Coordenadas*)_gerenciado)->enoughQuantity();

	remove->setEnabled(_points->count() > 0);
	add->setEnabled( enoughQuantity < 1 );
	emit valueChanged();
}
void CoordenadasOption::coordenadaChanged(double x, double y, double z, int pos) {
	((FabricaDeObjeto::Coordenadas*)_gerenciado)->value[pos] = Coordenada(x, y, z);
}

void NovoObjeto::optionValueChanged() {
	typedef FabricaDeObjeto::Type Type;
	typedef FabricaDeObjeto::Options::iterator iterator;
	for(iterator i = options.begin(); i != options.end(); ++i) {
		Type* atual = *i;
		if(!atual->fulfiled()){
			ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(false);
			return;
		}
	}
	ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(true);
}

QWidget* NovoObjeto::generateOptionsUi() {
	typedef FabricaDeObjeto::Type Type;
	typedef FabricaDeObjeto::Options::iterator iterator;

	QWidget* root = new QWidget();
	QGridLayout* rootLayout = new QGridLayout(root);

	int row=0;
	for(iterator i = options.begin(); i != options.end(); ++i) {
		Type* atual = *i;
		QLabel* label = new QLabel();
		label->setText( QString( atual->label.c_str() ) + ": " );
		rootLayout->addWidget(label, row, 0, 1, 1);

		switch(atual->getType()){
		case FabricaDeObjeto::STRING: {
			StringOption* nova = new StringOption(atual);
			QObject::connect(nova, SIGNAL(valueChanged()), this, SLOT(optionValueChanged()));
			rootLayout->addWidget(nova, row, 1, 1, 1);
		}
		break;
		case FabricaDeObjeto::COORDENADAS: {
			CoordenadasOption* nova = new CoordenadasOption(atual);
			QObject::connect(nova, SIGNAL(valueChanged()), this, SLOT(optionValueChanged()));
			rootLayout->addWidget(nova, ++row, 0, 1, 2);
		}
		break;
		case FabricaDeObjeto::COLOR: {
			ColorOption* nova = new ColorOption(atual);
			QObject::connect(nova, SIGNAL(valueChanged()), this, SLOT(optionValueChanged()));
			rootLayout->addWidget(nova, row, 1, 1, 1);
		}
		break;
		case FabricaDeObjeto::BOOLEAN: {
			BooleanOption* nova = new BooleanOption(atual);
			rootLayout->addWidget(nova, row, 1, 1, 1);
		}
		break;
		case FabricaDeObjeto::INT_INT: {
			IntIntsOption* nova = new IntIntsOption(atual);
			rootLayout->addWidget(nova, ++row, 0, 1, 2);
		}
		break;
		default:
		break;
		}

		row++;
	};

	return root;
};
