#include "viewport.h"

Viewport::Viewport()
{
	_janela = new Window(Coordenada(-10.0,-10.0), Coordenada(10.0,10.0), Coordenada(0.0,0.0,-10.0));
	_displayFile = new DisplayFile();
	_debugInformation = false;
}
Viewport::~Viewport()
{
}
#include <iostream>
void Viewport::persist(TiXmlNode* where) const {
	TiXmlElement *window = new TiXmlElement("window");
	_janela->persist(window);
	where->LinkEndChild( window );
	_displayFile->persist(where);
}
void Viewport::restore(TiXmlNode* where) {
	TiXmlNode *window = where->FirstChild("window");
	if(window != NULL)
		janela()->restore(window);
	else
		std::cerr << "Não há window especificada no arquivo XML, mantendo a window atual.";
	displayFile()->restore(where);
	_janela->atualizarCPP(*_displayFile);
}


void Viewport::showCoordinates(bool enable){
	_debugInformation = enable;
}

DisplayFile* Viewport::displayFile() {
	return _displayFile;
}

Window* Viewport::janela(){
	return _janela;
}

Coordenada Viewport::windowToViewport(const Coordenada& coordenada){
	/*
	 * Há uma função para cada eixo da coordenada original.
	 * fX(x) = aX * x + bX
	 * fY(y) = aY * y + bY
	 * fX(xMaximoMundo) = xMaximoWindow
	 * fX(xMinimoMundo) = xMinimoWindow
	 * fY(yMaximoMundo) = yMaximoWindow
	 * fY(yMinimoMundo) = yMinimoWindow
	 *
	 * Resolve-se o sistema linear para descobrir aX, bX, aY e bY.
	 * a = (MaximoWindow - MinimoWindow)/(MaximoMundo - MinimoMundo)
	 * b = MaximoWindow - a*MaximoMundo
	 */

	double xMaximoMundo = janela()->viewRightVector().modulo()/2;
//	double xMinimoMundo = -janela()->viewRightVector().modulo()/2;
	double xMaximoWindow = largura()-20;
	double xMinimoWindow = 20;
	double yMaximoMundo = janela()->viewUpVector().modulo()/2;
//	double yMinimoMundo = -janela()->viewUpVector().modulo()/2;
	double yMaximoWindow = 20;
	double yMinimoWindow = altura()-20;

	double bX, bY;
	double aX, aY;

	aX = (xMaximoWindow - xMinimoWindow)/(janela()->viewRightVector().modulo());
	bX = xMaximoWindow - xMaximoMundo*aX;
	aY = (yMaximoWindow - yMinimoWindow)/(janela()->viewUpVector().modulo());
	bY = yMaximoWindow - yMaximoMundo*aY;

	return Coordenada(round(coordenada.x()*aX + bX), round(coordenada.y()*aY + bY), coordenada.z());
}


void Viewport::repintar(){
	iniciarPintura();

	DisplayFile desenho = janela()->coordenadasDePlanoDeProjecao();

	for(std::vector<Objeto*>::const_iterator i= desenho.begin(); i != desenho.end(); ++i){
		Objeto* atual = *i;
		switch( atual->nivelDePreenchimento() ){
			case 0:
			repintar0(atual);
			break;
			case 1:
			repintar1(atual);
			break;
			case 2:
			repintar2(atual);
			break;
		}
		if(_debugInformation){
			pintarInformacoes(atual);
			repintar0(atual);
		}
	}


	finalizarPintura();
}
